#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf.urls import patterns, include, url
from views import *
from  django.views.generic.base import TemplateView

urlpatterns = patterns('',

    
    #Direct
    url(r'verify-with-pem-form', walk_chain_with_pem,  name="direct_verify_with_pem_form"),
    url(r'verify-with-discovery-form', walk_chain_with_discovery,  name="direct_verify_with_discovery_form"),
    
    
    url(r'^validate/(?P<endpoint>\S+)', validate_direct_endpoint,
        name="validate_direct_endpoint"),
    
    url(r'^get-certificate/(?P<endpoint>\S+).pem',
        get_direct_endpoint_certificate,
        name="get_direct_endpoint_certificate_pem"),
    
    url(r'^get-certificate/(?P<endpoint>\S+).txt',
        get_direct_endpoint_certificate,
        name="get_direct_endpoint_certificate_text"),
    
    url(r'^get-certificate/(?P<endpoint>\S+)',
        get_direct_endpoint_certificate,
        name="get_direct_endpoint_certificate_no_extension"),
    
    url(r'^$', TemplateView.as_view(template_name='direct/splash.html'),
        name="direct_splash"),

    )



from django.shortcuts import render
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from gdc.get_direct_certificate import DCert
import json
from utils import get_cert_highlights
from forms import DiscoverPEMForm, DiscoverPublicForm



def  walk_chain_with_pem(request):
    name = "Verify Certificate and Chain from a PEM x509 Certificate" 
    if request.method == 'POST':
        form = DiscoverPEMForm(request.POST)
        if form.is_valid():
            c = form.save()
            output_format = form.cleaned_data.get('output_format', "html")
            if output_format == "json":
               return HttpResponse(json.dumps(c, indent=4),
                                 content_type="application/json")
            #else its HTML
            highlights = get_cert_highlights(c)
            summary = highlights['Summary']
            details = highlights['Details']
            del highlights['Summary']
            del highlights['Details']
            
            for d in details:
                messages.warning(request, _(d))
            if not details:
                messages.success(request, _("Certificate looks good!"))
            return render_to_response('direct/discovery.html',
                                      {'name': name,
                                       'highlights': [highlights, ],
                                       'summary': summary},
                                            RequestContext(request))
        else:
            #The form is invalid
             return render_to_response('direct/generic/bootstrapform.html',
                                            {'form': form,
                                             'name': name},
                                            RequestContext(request))
   #this is a GET
    context= {'name':name, 'form': DiscoverPEMForm() }
    return render_to_response('direct/generic/bootstrapform.html',
                              RequestContext(request, context,))


def  walk_chain_with_discovery(request):
    number_of_certs =0
    certs =[]
    is_good = False
    is_unsure = False
    summary="Bad"
    name = "Verify the Certificate Chain from DNS and/or LDAP" 
    if request.method == 'POST':
        form = DiscoverPublicForm(request.POST)
        if form.is_valid():
            c = form.save()
            output_format = form.cleaned_data.get('output_format', "html")
            if output_format == "json":
               return HttpResponse(json.dumps(c, indent=4),
                                 content_type="application/json")
            #else its HTML
            if not c['is_found']:
                messages.error(request, _("The certificate was not found in LDAP or DNS."))
                return render_to_response('direct/discovery.html',
                                      {'name': name},
                                            RequestContext(request))
            if  c['dns']['status'] == 200:
                
                for i in c['dns']['cert_details']:
                    number_of_certs += 1
                    highlights = get_cert_highlights(i)
                    certs.append(highlights)
                    for d in highlights['Details']:
                        messages.warning(request, _(d))
                    if highlights['Summary'] == "Good":
                        is_good = True
                    if highlights['Summary'] == "Unsure":
                        is_unsure = True
            
            if  c['ldap']['status'] == 200:

                
                for i in c['ldap']['cert_details']:
                    number_of_certs += 1
                    highlights = get_cert_highlights(i)
                    certs.append(highlights)
                    for d in highlights['Details']:
                        messages.warning(request, _(d))
                    if highlights['Summary'] == "Good":
                        is_good = True
                    if highlights['Summary'] == "Unsure":
                        is_unsure = True
            
            if number_of_certs>1:
                messages.info(request, _("More than one certificate was discovered."))
            
            
            if is_unsure and not is_good:
                summary = "Unsure"
            if is_good:
                summary="Good"
            
            return render_to_response('direct/discovery.html',
                                      {'highlights': certs,
                                      'summary': summary
                                      },
                                            RequestContext(request))
            
            return HttpResponse(json.dumps(c, indent=4),
                         content_type="application/json")
        

        else:
            #The form is invalid
             return render_to_response('direct/generic/bootstrapform.html',
                                            {'form': form,
                                             'name': name},
                                            RequestContext(request))
   #this is a GET
    context= {'name':name, 'form': DiscoverPublicForm() }
    return render_to_response('direct/generic/bootstrapform.html',
                              RequestContext(request, context,))



def validate_direct_endpoint(request, endpoint):

    d = DCert(endpoint, endpoint)    
    result = d.validate_certificate()
    return HttpResponse(json.dumps(result),
                content_type="application/json")
    

def get_direct_endpoint_certificate(request, endpoint, file_extension="pem"):
    d = DCert(endpoint, endpoint)
    result = d.get_certificate()    
    response = HttpResponse(result, content_type="text/plain; charset=us-ascii") 
    
    if request.path.endswith(".pem") and result.__contains__("-----BEGIN CERTIFICATE-----"):
        response = HttpResponse(result, content_type="application/x-pem-file") 
    return response





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

import datetime
from  OpenSSL import crypto
from django.forms import  Form
from django import forms


from gdc.parse_certificate import build_chain
from gdc.get_direct_certificate import DCert
from django.utils.translation import ugettext_lazy as _

OUTPUT_CHOICES = (("json","JSON"),("html","HTML"))


class DiscoverPublicForm(Form):

    endpoint   = forms.CharField(label=_("Endpoint"),
                                    help_text="""
                                    Lookup in DNS and LDAP using this name. This is often the
                                    subject's common name ("CN") or the SubjectAltName's email
                                    address or DNS value.
                                    """)
    output_format    = forms.TypedChoiceField(choices=OUTPUT_CHOICES, initial="html")
    def clean_pem(self):
        endpoint = self.cleaned_data.get('endpoint', "")        
        return  endpoint.lstrip().rstrip() 
    
    required_css_class = 'required'
    
    def save(self, commit=True):
        
        endpoint = self.cleaned_data.get('endpoint', "")
        dc = DCert(endpoint=endpoint, expected_bound_entity=endpoint, save_to_disk=False)
        results = dc.validate_certificate()
        return results
        
 

class DiscoverPEMForm(Form):
    pem                     = forms.CharField(widget=forms.Textarea, label=_("PEM Certificate"),
                                    help_text="""
                                    This file's contents begins with -----BEGIN CERTIFICATE----- and ends with
                                    -----END CERTIFICATE-----.
                                    This file usually has the extension .pem or .crt and can be opened in any ASCII text editor. """)
    expected_bound_entity   = forms.CharField( required=False, label=_("Expected Bound Entity"),
                                    help_text="""
                                    This is often the subject's common name ("CN") the subject Alt Name's email address or DNS value.
                                    This  field is not required and present to facilitate checking that a certificate given is bound
                                    to the expected entity.
                                    """)
    output_format            = forms.TypedChoiceField(choices=OUTPUT_CHOICES, initial="html")
    
    
    
    
    def clean_pem(self):
        x509 = None
        pem = self.cleaned_data.get('pem', "")
        
        try:
            x509 = crypto.load_certificate(crypto.FILETYPE_PEM, pem)
        except:
            raise forms.ValidationError(_('Invalid PEM.'))

        if not x509:
            raise forms.ValidationError(_('Invalid PEM.'))
                
        return pem
    
    required_css_class = 'required'
    
    def save(self, commit=True):
        
        pem = self.cleaned_data.get('pem', "")
        expected_bound_entity = self.cleaned_data.get('expected_bound_entity', None)
        x509 = crypto.load_certificate(crypto.FILETYPE_PEM, pem)
        return build_chain(x509, expected_bound_entity)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import OrderedDict


def get_cert_highlights(certificate_results):
    
    
    san = False
    h =  OrderedDict()
    h['Summary'] = "Unsure"
    h['Details'] = []
    h['Common Name']= certificate_results['subject']['CN']
    h['Serial Number']= certificate_results['serial_number']
    h['Issuer']= certificate_results['issuer']['CN']
    h['Expired']= certificate_results['is_expired']
    h['Revocation Status']= certificate_results['revocation_status']
    h['Chain Status'] = certificate_results['chain_status']
    

    if certificate_results.has_key('bound_to_expected_entity'):
        h['Bound to Expected Entity']=  certificate_results['bound_to_expected_entity']
    else:
        h['Bound to Expected Entity']= "Unknown / No Expected Entity  Provided"
     
    extensions =   certificate_results.get('extensions', {}) 
    
    for k in extensions.keys():
        if k == "subjectAltNameDNS":
            h['Subject Alt Name DNS'] = extensions[k]
            h['Endpoint Type'] = "Domain Bound"
            san = True
        
        if k == "subjectAltNameemail":
            h['Subject Alt Name Email'] = extensions[k]
            h['Endpoint Type'] = "Email Bound"
            san = True
        
        if k == "basicConstraints":
            h['Basic Constraints'] = extensions[k]

        if k == "authorityKeyIdentifierkeyid":
            h['Authority Key Identifier Key ID'] = extensions[k]


        if k == "subjectKeyIdentifier":
            h['Subject Key Identifier'] = extensions[k]

        if k == "basicConstraints":
            h['Basic Constraints'] = extensions[k]
        
        
        
    h['Signature Algorithm'] = certificate_results['signature_algorithm']
    
    
    
    if h['Signature Algorithm'].__contains__('sha2') and \
       san == True and \
       h['Expired'] == False and \
       h['Revocation Status'] == "ACTIVE" and \
       h['Chain Status'] == "IN-TACT" and \
       h['Bound to Expected Entity'] == True:
            h['Summary'] = "Good"
    
    if h['Chain Status'] == "BROKEN":
        h['Summary'] = "Unsure"    
        h['Details'].append("One or more AIAs missing.")
        
    if h['Revocation Status'] == "REVOKED":
        h['Summary'] = "Bad"
        h['Details'].append("The certificate is revoked.")
    elif h['Revocation Status'] == "UNDETERMINED":
        h['Summary'] = "Unsure"
        h['Details'].append("One or more CRLs missing.")
    
    if h['Expired']== True:
        h['Summary'] = "Bad"
        h['Details'].append("The certificate is expired.")

    if h['Bound to Expected Entity']== False:
        h['Summary'] = "Bad"
        h['Details'].append("The certificate appears to be bound to the wrong entity. ") 

    
    return h
    



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-direct',
    version='0.7.5',
    packages=['direct', ],
    include_package_data=True,
    license='GPLv2',  
    description='A simple Django application that provides a RESTFul Direct certificate validation API.',
    long_description=README,
    url='https://github.com/videntity/django-direct',
    author='Alan Viars',
    author_email='sales@videntity.com',
    install_requires=[
        'getdc', 'Django>=1.7', 'django-widget-tweaks', 'django-bootstrap-form',
        'pyOpenSSL' ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
